@@ -145,6 +145,13 @@ sub fix_server_id {
   return $server_id;
 }
 
+sub fix_x_socket {
+  my $result = fix_socket(@_);
+  $result =~ s/mysqld\.([0-9]+)\.sock$/mysqlx\.$1\.sock/;
+
+  return $result;
+}
+
 sub fix_socket {
   my ($self, $config, $group_name, $group)= @_;
   # Put socket file in tmpdir
@@ -159,6 +166,7 @@ sub fix_socket {
     my $port = $group->value('port');
     $socket = "$dir/mysqld-$port.sock"; 
   }
+
   return $socket;
 }
 
@@ -268,6 +276,7 @@ my @mysqld_rules=
  { '#host' => \&fix_host },
  { 'port' => \&fix_port },
  { 'socket' => \&fix_socket },
+ { 'loose-mysqlx-socket' => \&fix_x_socket },
  { '#log-error' => \&fix_log_error },
  { 'general_log' => 1 },
  { 'general_log_file' => \&fix_log },
@@ -282,9 +291,9 @@ my @mysqld_rules=
  { 'ssl-cert' => \&fix_ssl_server_cert },
  { 'ssl-key' => \&fix_ssl_server_key },
  { 'loose-sha256_password_auto_generate_rsa_keys' => "0"},
- { 'early_plugin_load' => "" },
   );
 
+
 if (IS_WINDOWS)
 {
   # For simplicity, we use the same names for shared memory and 
@@ -510,12 +519,21 @@ sub resolve_at_variable {
   my $group_name=  join('.', @parts);
 
   $group_name =~ s/^\@//; # Remove at
-
-  my $from_group= $config->group($group_name)
-    or croak "There is no group named '$group_name' that ",
-      "can be used to resolve '$option_name'";
-
-  my $from= $from_group->value($option_name);
+  my $from;
+  
+  if ($group_name =~ "env")
+  {
+    $from = $ENV{$option_name};
+  } 
+  else
+  {
+    my $from_group= $config->group($group_name)
+      or croak "There is no group named '$group_name' that ",
+        "can be used to resolve '$option_name'";
+  
+    $from= $from_group->value($option_name);
+   }
+   
   $config->insert($group->name(), $option->name(), $from)
 }
 
