@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.
@@ -447,12 +447,13 @@ bool set_and_validate_user_attributes(THD *thd,
       }
     }
     /*
-      if there is a plugin specified with no auth string, then set
-      the account as expired.
+      if there is a plugin specified with no auth string, and that
+      plugin supports password expiration then set the account as expired.
     */
     if (Str->uses_identified_with_clause &&
         !(Str->uses_identified_by_clause ||
-        Str->uses_authentication_string_clause))
+        Str->uses_authentication_string_clause) &&
+        auth_plugin_supports_expiration(Str->plugin.str))
     {
       Str->alter_status.update_password_expired_column= true;
       what_to_set|= PASSWORD_EXPIRE_ATTR;
@@ -717,6 +718,7 @@ bool change_password(THD *thd, const char *host, const char *user,
   thd->lex->alter_password.expire_after_days= 0;
   thd->lex->alter_password.update_account_locked_column= false;
   thd->lex->alter_password.account_locked= false;
+  thd->lex->alter_password.update_password_expired_fields= false;
 
   /*
     When @@log-backward-compatible-user-definitions variable is ON
@@ -1270,6 +1272,7 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
   ulong what_to_update= 0;
   bool is_anonymous_user= false;
   bool rollback_whole_statement= false;
+  std::set<LEX_USER *> users_not_to_log;
   DBUG_ENTER("mysql_create_user");
 
   /*
@@ -1341,9 +1344,7 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
         result= true;
         break;
       }
-      else if (if_not_exists &&
-               (opt_general_log_raw
-               || !user_name->uses_identified_by_clause))
+      else if (if_not_exists)
       {
         String warn_user;
         append_user(thd, &warn_user, user_name, FALSE, FALSE);
@@ -1351,6 +1352,12 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
                             ER_USER_ALREADY_EXISTS,
                             ER_THD(thd, ER_USER_ALREADY_EXISTS),
                             warn_user.c_ptr_safe());
+        try
+        {
+          users_not_to_log.insert(tmp_user_name);
+        }
+        catch (...) {}
+        continue;
       }
      else
       {
@@ -1374,11 +1381,12 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
       my_error(ER_CANNOT_USER, MYF(0), "CREATE USER", wrong_users.c_ptr_safe());
   }
 
-  if (some_users_created || if_not_exists)
+  if (some_users_created ||
+      (if_not_exists && users_not_to_log.size() < list.elements))
   {
     String *rlb= &thd->rewritten_query;
     rlb->mem_free();
-    mysql_rewrite_create_alter_user(thd, rlb);
+    mysql_rewrite_create_alter_user(thd, rlb, &users_not_to_log);
 
     if (!thd->rewritten_query.length())
       result|= write_bin_log(thd, false, thd->query().str, thd->query().length,
@@ -1508,9 +1516,15 @@ bool mysql_drop_user(THD *thd, List <LEX_USER> &list, bool if_exists)
     my_error(ER_CANNOT_USER, MYF(0), "DROP USER", wrong_users.c_ptr_safe());
 
   if (some_users_deleted || if_exists)
-    result |= write_bin_log(thd, FALSE, thd->query().str, thd->query().length,
-                            transactional_tables);
-
+  {
+    int ret= commit_owned_gtid_by_partial_command(thd);
+    if (ret == 1)
+      result |= write_bin_log(thd, FALSE, thd->query().str,
+                              thd->query().length,
+                              transactional_tables);
+    else if (ret == -1)
+      result |= -1;
+  }
   lock.unlock();
 
   result|=
@@ -1688,6 +1702,7 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
   bool save_binlog_row_based;
   bool is_privileged_user= false;
   bool rollback_whole_statement= false;
+  std::set<LEX_USER *> users_not_to_log;
 
   DBUG_ENTER("mysql_alter_user");
 
@@ -1756,8 +1771,7 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
     if (!(acl_user= find_acl_user(user_from->host.str,
                                    user_from->user.str, TRUE)))
     {
-      if (if_exists && (opt_general_log_raw
-          || !user_from->uses_identified_by_clause))
+      if (if_exists)
       {
         String warn_user;
         append_user(thd, &warn_user, user_from, FALSE, FALSE);
@@ -1765,6 +1779,11 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
           ER_USER_DOES_NOT_EXIST,
           ER_THD(thd, ER_USER_DOES_NOT_EXIST),
           warn_user.c_ptr_safe());
+        try
+        {
+          users_not_to_log.insert(tmp_user_from);
+        }
+        catch (...) {}
       }
       else
       {
@@ -1776,6 +1795,9 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
       continue;
     }
 
+    if (user_from && user_from->plugin.str)
+      optimize_plugin_compare_by_pointer(&user_from->plugin);
+
     /* copy password expire attributes to individual lex user */
     user_from->alter_status= thd->lex->alter_password;
 
@@ -1840,12 +1862,13 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
       my_error(ER_CANNOT_USER, MYF(0), "ALTER USER", wrong_users.c_ptr_safe());
   }
 
-  if (some_user_altered || if_exists)
+  if (some_user_altered ||
+      (if_exists && users_not_to_log.size() < list.elements))
   {
     /* do query rewrite for ALTER USER */
     String *rlb= &thd->rewritten_query;
     rlb->mem_free();
-    mysql_rewrite_create_alter_user(thd, rlb);
+    mysql_rewrite_create_alter_user(thd, rlb, &users_not_to_log);
 
     result|= (write_bin_log(thd, false,
                             thd->rewritten_query.c_ptr_safe(),
