@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -887,7 +887,7 @@ bool lock_tablespace_name(THD *thd, const char *tablespace)
 // Function generating hash key for Tablespace_hash_set.
 extern "C" uchar *tablespace_set_get_key(const uchar *record,
                                          size_t *length,
-                                         my_bool not_used __attribute__((unused)))
+                                         my_bool not_used MY_ATTRIBUTE((unused)))
 {
   const char *tblspace_name= reinterpret_cast<const char *>(record);
   *length= strlen(tblspace_name);
@@ -1295,35 +1295,19 @@ void Global_backup_lock::release(THD *thd)
 
 void Global_backup_lock::set_explicit_locks_duration(THD *thd)
 {
-  bool should_own;
-
   DBUG_ENTER("Global_backup_lock::set_explicit_lock_duration");
 
   if (m_lock)
-  {
-    should_own= true;
     thd->mdl_context.set_lock_duration(m_lock, MDL_EXPLICIT);
-  }
-  else
-  {
-    should_own= false;
-  }
 
-  DBUG_ASSERT(should_own ==
+  DBUG_ASSERT((m_lock != NULL) ==
               thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
                                                            MDL_SHARED));
 
   if (m_prot_lock)
-  {
-    should_own= true;
     thd->mdl_context.set_lock_duration(m_prot_lock, MDL_EXPLICIT);
-  }
-  else
-  {
-    should_own= false;
-  }
 
-  DBUG_ASSERT(should_own ==
+  DBUG_ASSERT((m_prot_lock != NULL) ==
               thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
                                                      MDL_INTENTION_EXCLUSIVE));
 
