@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -230,6 +230,13 @@ public:
   */
   uint32 cur_log_old_open_count;
 
+  /*
+    If on init_info() call error_on_rli_init_info is true that means
+    that previous call to init_info() terminated with an error, RESET
+    SLAVE must be executed and the problem fixed manually.
+   */
+  bool error_on_rli_init_info;
+
   /*
     Let's call a group (of events) :
       - a transaction
@@ -295,6 +302,16 @@ public:
     gtid_set.ensure_sidno(sidno);
     gtid_set._add_gtid(sidno, gno);
   }
+
+  /**
+    Adds a GTID set to received GTID set.
+
+    @param gtid_set the gtid_set to add
+
+    @return RETURN_STATUS_OK or RETURN_STATUS_REPORTED_ERROR.
+  */
+  enum_return_status add_gtid_set(const Gtid_set *gtid_set);
+
   const Gtid_set *get_gtid_set() const { return &gtid_set; }
 
   int init_relay_log_pos(const char* log,
@@ -377,9 +394,7 @@ public:
     UNTIL_SQL_AFTER_GTIDS,
     UNTIL_SQL_AFTER_MTS_GAPS,
     UNTIL_SQL_VIEW_ID,
-#ifndef DBUG_OFF
     UNTIL_DONE
-#endif
   } until_condition;
   char until_log_name[FN_REFLEN];
   ulonglong until_log_pos;
@@ -1096,6 +1111,11 @@ public:
   */
   bool reported_unsafe_warning;
 
+  /*
+    'sql_thread_kill_accepted is set to TRUE when killed status is recognized.
+  */
+  bool sql_thread_kill_accepted;
+
   time_t get_row_stmt_start_timestamp()
   {
     return row_stmt_start_timestamp;
@@ -1258,12 +1278,6 @@ private:
   time_t row_stmt_start_timestamp;
   bool long_find_row_note_printed;
 
-  /*
-    If on init_info() call error_on_rli_init_info is true that means
-    that previous call to init_info() terminated with an error, RESET
-    SLAVE must be executed and the problem fixed manually.
-   */
-  bool error_on_rli_init_info;
 
  /**
    sets the suffix required for relay log names
