@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2007, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -139,7 +139,7 @@ end:
 /* Lookup function for my_hash tables with USER_STATS entries */
 static
 uchar *get_key_user_stats(USER_STATS *user_stats, size_t *length,
-			  my_bool not_used __attribute__((unused)))
+			  my_bool not_used MY_ATTRIBUTE((unused)))
 {
   *length= user_stats->user_len;
   return (uchar*) user_stats->user;
@@ -148,7 +148,7 @@ uchar *get_key_user_stats(USER_STATS *user_stats, size_t *length,
 /* Lookup function for my_hash tables with THREAD_STATS entries */
 static
 uchar *get_key_thread_stats(THREAD_STATS *thread_stats, size_t *length,
-			    my_bool not_used __attribute__((unused)))
+			    my_bool not_used MY_ATTRIBUTE((unused)))
 {
   *length= sizeof(my_thread_id);
   return (uchar *) &(thread_stats->id);
@@ -320,7 +320,7 @@ void init_global_thread_stats(void)
 
 static
 uchar *get_key_table_stats(TABLE_STATS *table_stats, size_t *length,
-			   my_bool not_used __attribute__((unused)))
+			   my_bool not_used MY_ATTRIBUTE((unused)))
 {
   *length= table_stats->table_len;
   return (uchar*) table_stats->table;
@@ -345,7 +345,7 @@ void init_global_table_stats(void)
 
 static
 uchar *get_key_index_stats(INDEX_STATS *index_stats, size_t *length,
-			   my_bool not_used __attribute__((unused)))
+			   my_bool not_used MY_ATTRIBUTE((unused)))
 {
   *length= index_stats->index_len;
   return (uchar*) index_stats->index;
@@ -899,7 +899,7 @@ void release_user_connection(THD *thd)
 */
 
 extern "C" uchar *get_key_conn(user_conn *buff, size_t *length,
-            my_bool not_used __attribute__((unused)))
+            my_bool not_used MY_ATTRIBUTE((unused)))
 {
   *length= buff->len;
   return (uchar*) buff->user;
@@ -1331,6 +1331,9 @@ static bool login_connection(THD *thd, bool extra_port_connection)
 void end_connection(THD *thd)
 {
   NET *net= thd->get_protocol_classic()->get_net();
+
+  mysql_audit_notify(thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_DISCONNECT), 0);
+
   plugin_thdvar_cleanup(thd, thd->m_enable_plugins);
 
   /*
@@ -1460,12 +1463,15 @@ bool thd_prepare_connection(THD *thd, bool extra_port_connection)
 
   @param thd        Thread handle.
   @param sql_errno  The error code to send before disconnect.
+  @param server_shutdown Argument passed to the THD's disconnect method.
+  @param generate_event  Generate Audit API disconnect event.
 
   @note
     For the connection that is doing shutdown, this is called twice
 */
 
-void close_connection(THD *thd, uint sql_errno, bool server_shutdown)
+void close_connection(THD *thd, uint sql_errno,
+                      bool server_shutdown, bool generate_event)
 {
   DBUG_ENTER("close_connection");
 
@@ -1481,8 +1487,11 @@ void close_connection(THD *thd, uint sql_errno, bool server_shutdown)
     sleep(0); /* Workaround to avoid tailcall optimisation */
   }
 
-  mysql_audit_notify(thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_DISCONNECT),
-                     sql_errno);
+  if (generate_event)
+    mysql_audit_notify(thd,
+                       AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_DISCONNECT),
+                       sql_errno);
+
   DBUG_VOID_RETURN;
 }
 
