@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2016, Percona Inc. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -167,7 +167,7 @@ buf_dblwr_init(
 Creates the doublewrite buffer to a new InnoDB installation. The header of the
 doublewrite buffer is placed on the trx system header page.
 @return true if successful, false if not. */
-__attribute__((warn_unused_result))
+MY_ATTRIBUTE((warn_unused_result))
 bool
 buf_dblwr_create(void)
 /*==================*/
@@ -338,7 +338,7 @@ start_again:
 }
 
 /** Compute the path to the parallel doublewrite buffer, if not already done */
-__attribute__((warn_unused_result))
+MY_ATTRIBUTE((warn_unused_result))
 static
 dberr_t
 buf_parallel_dblwr_make_path(void)
@@ -827,7 +827,7 @@ buf_dblwr_process(void)
 				true, read_buf, page_size,
 				fsp_is_checksum_disabled(space_id))) {
 
-				ib::warn() << "Database page corruption or"
+				ib::info() << "Database page corruption or"
 					<< " a failed file read of page "
 					<< page_id
 					<< ". Trying to recover it from the"
@@ -1479,7 +1479,7 @@ retry:
 /** Compute the size and path of the parallel doublewrite buffer, create it,
 and disable OS caching for it
 @return DB_SUCCESS or error code */
-static __attribute__((warn_unused_result))
+static MY_ATTRIBUTE((warn_unused_result))
 dberr_t
 buf_parallel_dblwr_file_create(void)
 {
@@ -1514,6 +1514,14 @@ buf_parallel_dblwr_file_create(void)
 			ib::error() << "A parallel doublewrite file "
 				    << parallel_dblwr_buf.path
 				    << " found on startup.";
+			if (srv_force_recovery == SRV_FORCE_NO_LOG_REDO) {
+				ib::error() << "Since --innodb-force-recovery "
+					"is set to 6, which skips doublewrite "
+					"buffer recovery, please move away "
+					"the file above and restore it before "
+					"attempting a lower forced recovery "
+					"setting";
+			}
 		}
 		return(DB_ERROR);
 	}
@@ -1524,7 +1532,7 @@ buf_parallel_dblwr_file_create(void)
 	success = os_file_set_size(parallel_dblwr_buf.path,
 				   parallel_dblwr_buf.file, size, false);
 	if (!success) {
-		buf_parallel_dblwr_destroy();
+		buf_parallel_dblwr_free(true);
 		return(DB_ERROR);
 	}
 	ut_ad(os_file_get_size(parallel_dblwr_buf.file) == size);
@@ -1567,7 +1575,7 @@ buf_parallel_dblwr_create(void)
 						       * UNIV_PAGE_SIZE,
 					mem_key_parallel_doublewrite));
 		if (!dblwr_shard->write_buf_unaligned) {
-			buf_parallel_dblwr_destroy();
+			buf_parallel_dblwr_free(true);
 			return(DB_OUT_OF_MEMORY);
 		}
 		dblwr_shard->write_buf = static_cast<byte*>(
@@ -1579,7 +1587,7 @@ buf_parallel_dblwr_create(void)
 				  * sizeof(void*),
 				  mem_key_parallel_doublewrite));
 		if (!dblwr_shard->buf_block_arr) {
-			buf_parallel_dblwr_destroy();
+			buf_parallel_dblwr_free(true);
 			return(DB_OUT_OF_MEMORY);
 		}
 
@@ -1591,10 +1599,11 @@ buf_parallel_dblwr_create(void)
 	return(DB_SUCCESS);
 }
 
-/** Close and delete the doublewrite buffer file and free its memory data
-structure. */
+/** Cleanup parallel doublewrite memory structures and optionally close and
+delete the doublewrite buffer file too.
+@param	delete_file	whether to close and delete the buffer file too  */
 void
-buf_parallel_dblwr_destroy(void)
+buf_parallel_dblwr_free(bool delete_file)
 {
 	for (ulint i = 0; i < buf_parallel_dblwr_shard_num(); i++) {
 
@@ -1610,8 +1619,10 @@ buf_parallel_dblwr_destroy(void)
 		ut_free(dblwr_shard->buf_block_arr);
 	}
 
-	buf_parallel_dblwr_close();
-	buf_parallel_dblwr_delete();
+	if (delete_file) {
+		buf_parallel_dblwr_close();
+		buf_parallel_dblwr_delete();
+	}
 
 	ut_free(parallel_dblwr_buf.path);
 	parallel_dblwr_buf.path = NULL;
