@@ -348,7 +348,7 @@ dict_create_sys_virtual_tuple(
 /***************************************************************//**
 Builds a table definition to insert.
 @return DB_SUCCESS or error code */
-static __attribute__((nonnull, warn_unused_result))
+static MY_ATTRIBUTE((nonnull, warn_unused_result))
 dberr_t
 dict_build_table_def_step(
 /*======================*/
@@ -851,7 +851,7 @@ dict_create_search_tuple(
 /***************************************************************//**
 Builds an index definition row to insert.
 @return DB_SUCCESS or error code */
-static __attribute__((nonnull, warn_unused_result))
+static MY_ATTRIBUTE((nonnull, warn_unused_result))
 dberr_t
 dict_build_index_def_step(
 /*======================*/
@@ -968,7 +968,7 @@ dict_build_field_def_step(
 /***************************************************************//**
 Creates an index tree for the index if it is not a member of a cluster.
 @return DB_SUCCESS or DB_OUT_OF_FILE_SPACE */
-static __attribute__((nonnull, warn_unused_result))
+static MY_ATTRIBUTE((nonnull, warn_unused_result))
 dberr_t
 dict_create_index_tree_step(
 /*========================*/
@@ -1999,7 +1999,7 @@ dict_create_or_check_sys_virtual()
 /****************************************************************//**
 Evaluate the given foreign key SQL statement.
 @return error code or DB_SUCCESS */
-static __attribute__((nonnull, warn_unused_result))
+static MY_ATTRIBUTE((nonnull, warn_unused_result))
 dberr_t
 dict_foreign_eval_sql(
 /*==================*/
@@ -2064,7 +2064,7 @@ dict_foreign_eval_sql(
 Add a single foreign key field definition to the data dictionary tables in
 the database.
 @return error code or DB_SUCCESS */
-static __attribute__((nonnull, warn_unused_result))
+static MY_ATTRIBUTE((nonnull, warn_unused_result))
 dberr_t
 dict_create_add_foreign_field_to_dictionary(
 /*========================================*/
@@ -2233,25 +2233,56 @@ dict_foreign_has_col_as_base_col(
 	return(false);
 }
 
-/** Check if a foreign constraint is on columns served as based columns
-of some virtual column, or the column is part of virtual index (index
-that contains virtual column). This is to prevent creating SET NULL or
-CASCADE constraint on such columns
+/** Check if a foreign constraint is on the given column name.
+@param[in]	col_name	column name to be searched for fk constraint
+@param[in]	table		table to which foreign key constraint belongs
+@return true if fk constraint is present on the table, false otherwise. */
+static
+bool
+dict_foreign_base_for_stored(
+	const char*		col_name,
+	const dict_table_t*	table)
+{
+	/* Loop through each stored column and check if its base column has
+	the same name as the column name being checked */
+	dict_s_col_list::const_iterator	it;
+	for (it = table->s_cols->begin();
+	     it != table->s_cols->end(); ++it) {
+		dict_s_col_t	s_col = *it;
+
+		for (ulint j = 0; j < s_col.num_base; j++) {
+			if (strcmp(col_name, dict_table_get_col_name(
+						table,
+						s_col.base_col[j]->ind)) == 0) {
+				return(true);
+			}
+		}
+	}
+
+	return(false);
+}
+
+/** Check if a foreign constraint is on columns served as base columns
+of any stored column. This is to prevent creating SET NULL or CASCADE
+constraint on such columns
 @param[in]	local_fk_set	set of foreign key objects, to be added to
 the dictionary tables
 @param[in]	table		table to which the foreign key objects in
 local_fk_set belong to
 @return true if yes, otherwise, false */
 bool
-dict_foreigns_has_v_base_col(
+dict_foreigns_has_s_base_col(
 	const dict_foreign_set&	local_fk_set,
 	const dict_table_t*	table)
 {
 	dict_foreign_t*	foreign;
 
+	if (table->s_cols == NULL) {
+		return (false);
+	}
+
 	for (dict_foreign_set::const_iterator it = local_fk_set.begin();
-	     it != local_fk_set.end();
-	     ++it) {
+	     it != local_fk_set.end(); ++it) {
 
 		foreign = *it;
 		ulint	type = foreign->type;
@@ -2265,20 +2296,14 @@ dict_foreigns_has_v_base_col(
 
 		for (ulint i = 0; i < foreign->n_fields; i++) {
 			/* Check if the constraint is on a column that
-			is a base column of some virtual column */
-			if (dict_foreign_has_col_as_base_col(
-				    foreign->foreign_col_names[i], table)) {
-				return(true);
-			}
-
-			/* Check if the column is part of a virtual index (
-			index contains virtual columns) */
-			if (dict_foreign_has_col_in_v_index(
+			is a base column of any stored column */
+			if (dict_foreign_base_for_stored(
 				foreign->foreign_col_names[i], table)) {
 				return(true);
 			}
 		}
 	}
+
 	return(false);
 }
 
@@ -2319,29 +2344,6 @@ dict_foreigns_has_this_col(
 	return(false);
 }
 
-/** Check if a virtual column could have base columns in foreign constraints,
-this is to prevent creating virtual index on such column
-@param[in]	table	table
-@param[in]	v_col_n	virtual column number for the virtual column to check
-@return true if the virtual column could have base columns in constraint */
-bool
-dict_table_has_base_in_foreign(
-	const dict_table_t*	table,
-	ulint			v_col_n)
-{
-	const dict_v_col_t*	v_col = dict_table_get_nth_v_col(table, v_col_n);
-
-	for (ulint j = 0; j < v_col->num_base; j++) {
-		if (dict_foreigns_has_this_col(
-			    table, dict_table_get_col_name(
-				    table, v_col->base_col[j]->ind))) {
-			return(true);
-		}
-	}
-
-	return(false);
-}
-
 /** Adds the given set of foreign key objects to the dictionary tables
 in the database. This function does not modify the dictionary cache. The
 caller must ensure that all foreign key objects contain a valid constraint
