@@ -75,6 +75,7 @@ extern mysql_pfs_key_t	ibuf_mutex_key;
 extern mysql_pfs_key_t	ibuf_pessimistic_insert_mutex_key;
 extern mysql_pfs_key_t	log_bmp_sys_mutex_key;
 extern mysql_pfs_key_t	log_sys_mutex_key;
+extern mysql_pfs_key_t	log_sys_write_mutex_key;
 extern mysql_pfs_key_t	log_cmdq_mutex_key;
 extern mysql_pfs_key_t	log_flush_order_mutex_key;
 extern mysql_pfs_key_t	mutex_list_mutex_key;
@@ -140,9 +141,6 @@ extern	mysql_pfs_key_t	dict_table_stats_key;
 extern  mysql_pfs_key_t trx_sys_rw_lock_key;
 extern  mysql_pfs_key_t hash_table_locks_key;
 extern  mysql_pfs_key_t master_key_id_mutex_key;
-# ifdef UNIV_DEBUG
-extern	mysql_pfs_key_t buf_chunk_map_latch_key;
-# endif /* UNIV_DEBUG */
 #endif /* UNIV_PFS_RWLOCK */
 
 /** Prints info of the sync system.
