@@ -201,20 +201,20 @@ log_buffer_extend(
 	ulint	move_end;
 	byte*	tmp_buf[OS_FILE_LOG_BLOCK_SIZE];
 
-	log_mutex_enter();
+	log_mutex_enter_all();
 
 	while (log_sys->is_extending) {
 		/* Another thread is trying to extend already.
 		Needs to wait for. */
-		log_mutex_exit();
+		log_mutex_exit_all();
 
 		log_buffer_flush_to_disk();
 
-		log_mutex_enter();
+		log_mutex_enter_all();
 
 		if (srv_log_buffer_size > len / UNIV_PAGE_SIZE) {
 			/* Already extended enough by the others */
-			log_mutex_exit();
+			log_mutex_exit_all();
 			return;
 		}
 	}
@@ -236,11 +236,11 @@ log_buffer_extend(
 	       != ut_calc_align_down(log_sys->buf_next_to_write,
 				     OS_FILE_LOG_BLOCK_SIZE)) {
 		/* Buffer might have >1 blocks to write still. */
-		log_mutex_exit();
+		log_mutex_exit_all();
 
 		log_buffer_flush_to_disk();
 
-		log_mutex_enter();
+		log_mutex_enter_all();
 	}
 
 	move_start = ut_calc_align_down(
@@ -258,11 +258,17 @@ log_buffer_extend(
 	/* reallocate log buffer */
 	srv_log_buffer_size = len / UNIV_PAGE_SIZE + 1;
 	ut_free(log_sys->buf_ptr);
-	log_sys->buf_ptr = static_cast<byte*>(
-		ut_zalloc_nokey(LOG_BUFFER_SIZE + OS_FILE_LOG_BLOCK_SIZE));
-	log_sys->buf = static_cast<byte*>(
-		ut_align(log_sys->buf_ptr, OS_FILE_LOG_BLOCK_SIZE));
+
 	log_sys->buf_size = LOG_BUFFER_SIZE;
+
+	log_sys->buf_ptr = static_cast<byte*>(
+		ut_zalloc_nokey(log_sys->buf_size * 2
+				+ MAX_SRV_LOG_WRITE_AHEAD_SIZE));
+	log_sys->buf = static_cast<byte*>(
+		ut_align(log_sys->buf_ptr, MAX_SRV_LOG_WRITE_AHEAD_SIZE));
+
+	log_sys->first_in_use = true;
+
 	log_sys->max_buf_free = log_sys->buf_size / LOG_BUF_FLUSH_RATIO
 		- LOG_BUF_FLUSH_MARGIN;
 
@@ -272,7 +278,7 @@ log_buffer_extend(
 	ut_ad(log_sys->is_extending);
 	log_sys->is_extending = false;
 
-	log_mutex_exit();
+	log_mutex_exit_all();
 
 	ib::info() << "innodb_log_buffer_size was extended to "
 		<< LOG_BUFFER_SIZE << ".";
@@ -522,8 +528,6 @@ log_close(void)
 	ulint		first_rec_group;
 	lsn_t		oldest_lsn;
 	lsn_t		lsn;
-	lsn_t		tracked_lsn;
-	lsn_t		tracked_lsn_age;
 	log_t*		log	= log_sys;
 	lsn_t		checkpoint_age;
 
@@ -554,8 +558,9 @@ log_close(void)
 
 	if (srv_track_changed_pages) {
 
-		tracked_lsn = log_get_tracked_lsn();
-		tracked_lsn_age = lsn - tracked_lsn;
+		lsn_t tracked_lsn = log_get_tracked_lsn();
+		ut_ad(tracked_lsn > 0);
+		lsn_t tracked_lsn_age = lsn - tracked_lsn;
 
 		if (tracked_lsn_age >= log->log_group_capacity) {
 
@@ -614,7 +619,9 @@ log_group_get_capacity(
 /*===================*/
 	const log_group_t*	group)	/*!< in: log group */
 {
-	ut_ad(log_mutex_own());
+	/* The lsn parameters are updated while holding both the mutexes
+	and it is ok to have either of them while reading */
+	ut_ad(log_mutex_own() || log_write_mutex_own());
 
 	return((group->file_size - LOG_FILE_HDR_SIZE) * group->n_files);
 }
@@ -631,7 +638,9 @@ log_group_calc_size_offset(
 					log group */
 	const log_group_t*	group)	/*!< in: log group */
 {
-	ut_ad(log_mutex_own());
+	/* The lsn parameters are updated while holding both the mutexes
+	and it is ok to have either of them while reading */
+	ut_ad(log_mutex_own() || log_write_mutex_own());
 
 	return(offset - LOG_FILE_HDR_SIZE * (1 + offset / group->file_size));
 }
@@ -648,7 +657,9 @@ log_group_calc_real_offset(
 					log group */
 	const log_group_t*	group)	/*!< in: log group */
 {
-	ut_ad(log_mutex_own());
+	/* The lsn parameters are updated while holding both the mutexes
+	and it is ok to have either of them while reading */
+	ut_ad(log_mutex_own() || log_write_mutex_own());
 
 	return(offset + LOG_FILE_HDR_SIZE
 	       * (1 + offset / (group->file_size - LOG_FILE_HDR_SIZE)));
@@ -669,11 +680,14 @@ log_group_calc_lsn_offset(
 	lsn_t	group_size;
 	lsn_t	offset;
 
-	ut_ad(log_mutex_own());
+	/* The lsn parameters are updated while holding both the mutexes
+	and it is ok to have either of them while reading */
+	ut_ad(log_mutex_own() || log_write_mutex_own());
 
 	gr_lsn = group->lsn;
 
-	gr_lsn_size_offset = log_group_calc_size_offset(group->lsn_offset, group);
+	gr_lsn_size_offset = log_group_calc_size_offset(
+		group->lsn_offset, group);
 
 	group_size = log_group_get_capacity(group);
 
@@ -760,7 +774,7 @@ and lsn - buf_get_oldest_modification().
 @retval true on success
 @retval false if the smallest log group is too small to
 accommodate the number of OS threads in the database server */
-static __attribute__((warn_unused_result))
+static MY_ATTRIBUTE((warn_unused_result))
 bool
 log_calc_max_ages(void)
 /*===================*/
@@ -846,6 +860,7 @@ log_init(void)
 	log_sys = static_cast<log_t*>(ut_zalloc_nokey(sizeof(log_t)));
 
 	mutex_create(LATCH_ID_LOG_SYS, &log_sys->mutex);
+	mutex_create(LATCH_ID_LOG_WRITE, &log_sys->write_mutex);
 
 	mutex_create(LATCH_ID_LOG_FLUSH_ORDER, &log_sys->log_flush_order_mutex);
 
@@ -857,14 +872,16 @@ log_init(void)
 	ut_a(LOG_BUFFER_SIZE >= 16 * OS_FILE_LOG_BLOCK_SIZE);
 	ut_a(LOG_BUFFER_SIZE >= 4 * UNIV_PAGE_SIZE);
 
-	log_sys->buf_ptr = static_cast<byte*>(
-		ut_zalloc_nokey(LOG_BUFFER_SIZE + OS_FILE_LOG_BLOCK_SIZE));
-
-	log_sys->buf = static_cast<byte*>(
-		ut_align(log_sys->buf_ptr, OS_FILE_LOG_BLOCK_SIZE));
-
 	log_sys->buf_size = LOG_BUFFER_SIZE;
 
+	log_sys->buf_ptr = static_cast<byte*>(
+		ut_zalloc_nokey(log_sys->buf_size * 2
+				+ MAX_SRV_LOG_WRITE_AHEAD_SIZE));
+	log_sys->buf = static_cast<byte*>(
+		ut_align(log_sys->buf_ptr, MAX_SRV_LOG_WRITE_AHEAD_SIZE));
+
+	log_sys->first_in_use = true;
+
 	log_sys->max_buf_free = log_sys->buf_size / LOG_BUF_FLUSH_RATIO
 		- LOG_BUF_FLUSH_MARGIN;
 	log_sys->check_flush_or_checkpoint = true;
@@ -883,16 +900,19 @@ log_init(void)
 	/*----------------------------*/
 
 	log_sys->last_checkpoint_lsn = log_sys->lsn;
+	log_sys->next_checkpoint_lsn = log_sys->lsn;
 
 	rw_lock_create(
 		checkpoint_lock_key, &log_sys->checkpoint_lock,
 		SYNC_NO_ORDER_CHECK);
 
 	log_sys->checkpoint_buf_ptr = static_cast<byte*>(
-		ut_zalloc_nokey(2 * OS_FILE_LOG_BLOCK_SIZE));
+		ut_zalloc_nokey(OS_FILE_LOG_BLOCK_SIZE
+				+ MAX_SRV_LOG_WRITE_AHEAD_SIZE));
 
 	log_sys->checkpoint_buf = static_cast<byte*>(
-		ut_align(log_sys->checkpoint_buf_ptr, OS_FILE_LOG_BLOCK_SIZE));
+		ut_align(log_sys->checkpoint_buf_ptr,
+			 MAX_SRV_LOG_WRITE_AHEAD_SIZE));
 
 	/*----------------------------*/
 
@@ -909,7 +929,7 @@ log_init(void)
 /******************************************************************//**
 Inits a log group to the log system.
 @return true if success, false if not */
-__attribute__((warn_unused_result))
+MY_ATTRIBUTE((warn_unused_result))
 bool
 log_group_init(
 /*===========*/
@@ -943,57 +963,26 @@ log_group_init(
 	for (i = 0; i < n_files; i++) {
 		group->file_header_bufs_ptr[i] = static_cast<byte*>(
 			ut_zalloc_nokey(LOG_FILE_HDR_SIZE
-					+ OS_FILE_LOG_BLOCK_SIZE));
+					+ MAX_SRV_LOG_WRITE_AHEAD_SIZE));
 
 		group->file_header_bufs[i] = static_cast<byte*>(
 			ut_align(group->file_header_bufs_ptr[i],
-				 OS_FILE_LOG_BLOCK_SIZE));
+				 MAX_SRV_LOG_WRITE_AHEAD_SIZE));
 	}
 
 	group->checkpoint_buf_ptr = static_cast<byte*>(
-		ut_zalloc_nokey(2 * OS_FILE_LOG_BLOCK_SIZE));
+		ut_zalloc_nokey(OS_FILE_LOG_BLOCK_SIZE +
+				MAX_SRV_LOG_WRITE_AHEAD_SIZE));
 
 	group->checkpoint_buf = static_cast<byte*>(
-		ut_align(group->checkpoint_buf_ptr,OS_FILE_LOG_BLOCK_SIZE));
+		ut_align(group->checkpoint_buf_ptr,
+			 MAX_SRV_LOG_WRITE_AHEAD_SIZE));
 
 	UT_LIST_ADD_LAST(log_sys->log_groups, group);
 
 	return(log_calc_max_ages());
 }
 #endif /* !UNIV_HOTBACKUP */
-/******************************************************//**
-Update log_sys after write completion. */
-static
-void
-log_sys_write_completion(void)
-/*==========================*/
-{
-	ulint	move_start;
-	ulint	move_end;
-
-	ut_ad(log_mutex_own());
-
-	log_sys->write_lsn = log_sys->lsn;
-	log_sys->buf_next_to_write = log_sys->write_end_offset;
-
-	if (log_sys->write_end_offset > log_sys->max_buf_free / 2) {
-		/* Move the log buffer content to the start of the
-		buffer */
-
-		move_start = ut_calc_align_down(
-			log_sys->write_end_offset,
-			OS_FILE_LOG_BLOCK_SIZE);
-		move_end = ut_calc_align(log_sys->buf_free,
-					 OS_FILE_LOG_BLOCK_SIZE);
-
-		ut_memmove(log_sys->buf, log_sys->buf + move_start,
-			   move_end - move_start);
-		log_sys->buf_free -= move_start;
-
-		log_sys->buf_next_to_write -= move_start;
-	}
-}
-
 /******************************************************//**
 Completes an i/o to a log file. */
 void
@@ -1048,7 +1037,7 @@ log_group_file_header_flush(
 	byte*	buf;
 	lsn_t	dest_offset;
 
-	ut_ad(log_mutex_own());
+	ut_ad(log_write_mutex_own());
 	ut_ad(!recv_no_log_write);
 	ut_ad(group->id == 0);
 	ut_a(nth_file < group->n_files);
@@ -1128,7 +1117,7 @@ log_group_write_buf(
 	lsn_t		next_offset;
 	ulint		i;
 
-	ut_ad(log_mutex_own());
+	ut_ad(log_write_mutex_own());
 	ut_ad(!recv_no_log_write);
 	ut_a(len % OS_FILE_LOG_BLOCK_SIZE == 0);
 	ut_a(start_lsn % OS_FILE_LOG_BLOCK_SIZE == 0);
@@ -1249,6 +1238,41 @@ log_write_flush_to_disk_low()
 	os_event_set(log_sys->flush_event);
 }
 
+/** Switch the log buffer in use, and copy the content of last block
+from old log buffer to the head of the to be used one. Thus, buf_free and
+buf_next_to_write would be changed accordingly */
+static inline
+void
+log_buffer_switch()
+{
+	ut_ad(log_mutex_own());
+	ut_ad(log_write_mutex_own());
+
+	const byte*	old_buf = log_sys->buf;
+	ulint		area_end = ut_calc_align(log_sys->buf_free,
+						 OS_FILE_LOG_BLOCK_SIZE);
+
+	if (log_sys->first_in_use) {
+		ut_ad((reinterpret_cast<uintptr_t>(log_sys->buf)
+		       % srv_log_write_ahead_size) == 0);
+		log_sys->buf += log_sys->buf_size;
+	} else {
+		log_sys->buf -= log_sys->buf_size;
+		ut_ad((reinterpret_cast<uintptr_t>(log_sys->buf)
+		       % srv_log_write_ahead_size) == 0);
+	}
+
+	log_sys->first_in_use = !log_sys->first_in_use;
+
+	/* Copy the last block to new buf */
+	ut_memcpy(log_sys->buf,
+		  old_buf + area_end - OS_FILE_LOG_BLOCK_SIZE,
+		  OS_FILE_LOG_BLOCK_SIZE);
+
+	log_sys->buf_free %= OS_FILE_LOG_BLOCK_SIZE;
+	log_sys->buf_next_to_write = log_sys->buf_free;
+}
+
 /** Ensure that the log has been written to the log file up to a given
 log entry (such as that of a transaction commit). Start a new write, or
 wait and check if an already running write is covering the request.
@@ -1264,6 +1288,8 @@ log_write_up_to(
 #ifdef UNIV_DEBUG
 	ulint		loop_count	= 0;
 #endif /* UNIV_DEBUG */
+	byte*           write_buf;
+	lsn_t           write_lsn;
 
 	ut_ad(!srv_read_only_mode);
 
@@ -1289,7 +1315,7 @@ loop:
 	}
 #endif
 
-	log_mutex_enter();
+	log_write_mutex_enter();
 	ut_ad(!recv_no_log_write);
 
 	lsn_t	limit_lsn = flush_to_disk
@@ -1297,7 +1323,7 @@ loop:
 		: log_sys->write_lsn;
 
 	if (limit_lsn >= lsn) {
-		log_mutex_exit();
+		log_write_mutex_exit();
 		return;
 	}
 
@@ -1306,7 +1332,7 @@ loop:
 	/* write requests during fil_flush() might not be good for Windows */
 	if (log_sys->n_pending_flushes > 0
 	    || !os_event_is_set(log_sys->flush_event)) {
-		log_mutex_exit();
+		log_write_mutex_exit();
 		os_event_wait(log_sys->flush_event);
 		goto loop;
 	}
@@ -1330,7 +1356,7 @@ loop:
 		for us. */
 		bool work_done = log_sys->current_flush_lsn >= lsn;
 
-		log_mutex_exit();
+		log_write_mutex_exit();
 
 		os_event_wait(log_sys->flush_event);
 
@@ -1341,10 +1367,11 @@ loop:
 		}
 	}
 
+	log_mutex_enter();
 	if (!flush_to_disk
 	    && log_sys->buf_free == log_sys->buf_next_to_write) {
 		/* Nothing to write and no flush to disk requested */
-		log_mutex_exit();
+		log_mutex_exit_all();
 		return;
 	}
 
@@ -1368,14 +1395,12 @@ loop:
 
 		if (log_sys->buf_free == log_sys->buf_next_to_write) {
 			/* Nothing to write, flush only */
-			log_mutex_exit();
+			log_mutex_exit_all();
 			log_write_flush_to_disk_low();
 			return;
 		}
 	}
 
-	group = UT_LIST_GET_FIRST(log_sys->log_groups);
-
 	start_offset = log_sys->buf_next_to_write;
 	end_offset = log_sys->buf_free;
 
@@ -1389,8 +1414,17 @@ loop:
 		log_sys->buf + area_end - OS_FILE_LOG_BLOCK_SIZE,
 		log_sys->next_checkpoint_no);
 
+	write_lsn = log_sys->lsn;
+	write_buf = log_sys->buf;
+
+	log_buffer_switch();
+
 	group = UT_LIST_GET_FIRST(log_sys->log_groups);
 
+	log_group_set_fields(group, log_sys->write_lsn);
+
+	log_mutex_exit();
+
 	/* Calculate pad_size if needed. */
 	pad_size = 0;
 	if (write_ahead_size > OS_FILE_LOG_BLOCK_SIZE) {
@@ -1398,7 +1432,7 @@ loop:
 		ulint	end_offset_in_unit;
 
 		end_offset = log_group_calc_lsn_offset(
-			ut_uint64_align_up(log_sys->lsn,
+			ut_uint64_align_up(write_lsn,
 					   OS_FILE_LOG_BLOCK_SIZE),
 			group);
 		end_offset_in_unit = (ulint) (end_offset % write_ahead_size);
@@ -1414,13 +1448,13 @@ loop:
 				pad_size = log_sys->buf_size - area_end;
 			}
 
-			::memset(log_sys->buf + area_end, 0, pad_size);
+			::memset(write_buf + area_end, 0, pad_size);
 		}
 	}
 
 	/* Do the write to the log files */
 	log_group_write_buf(
-		group, log_sys->buf + area_start,
+		group, write_buf + area_start,
 		area_end - area_start + pad_size,
 #ifdef UNIV_DEBUG
 		pad_size,
@@ -1431,11 +1465,7 @@ loop:
 
 	srv_stats.log_padded.add(pad_size);
 
-	log_sys->write_end_offset = log_sys->buf_free;
-
-	log_group_set_fields(group, log_sys->write_lsn);
-
-	log_sys_write_completion();
+	log_sys->write_lsn = write_lsn;
 
 #ifndef _WIN32
 	if (srv_unix_file_flush_method == SRV_UNIX_O_DSYNC
@@ -1447,7 +1477,7 @@ loop:
 	}
 #endif /* !_WIN32 */
 
-	log_mutex_exit();
+	log_write_mutex_exit();
 
 	if (flush_to_disk) {
 		log_write_flush_to_disk_low();
@@ -1571,6 +1601,7 @@ log_complete_checkpoint(void)
 
 	log_sys->next_checkpoint_no++;
 
+	ut_ad(log_sys->next_checkpoint_lsn >= log_sys->last_checkpoint_lsn);
 	log_sys->last_checkpoint_lsn = log_sys->next_checkpoint_lsn;
 	MONITOR_SET(MONITOR_LSN_CHECKPOINT_AGE,
 		    log_sys->lsn - log_sys->last_checkpoint_lsn);
@@ -1623,6 +1654,7 @@ log_group_checkpoint(
 
 	ut_ad(!srv_read_only_mode);
 	ut_ad(log_mutex_own());
+	ut_ad(srv_shutdown_state != SRV_SHUTDOWN_LAST_PHASE);
 #if LOG_CHECKPOINT_SIZE > OS_FILE_LOG_BLOCK_SIZE
 # error "LOG_CHECKPOINT_SIZE > OS_FILE_LOG_BLOCK_SIZE"
 #endif
@@ -1636,6 +1668,11 @@ log_group_checkpoint(
 	buf = group->checkpoint_buf;
 	memset(buf, 0, OS_FILE_LOG_BLOCK_SIZE);
 
+#ifdef UNIV_DEBUG
+	lsn_t		old_next_checkpoint_lsn
+		= mach_read_from_8(buf + LOG_CHECKPOINT_LSN);
+	ut_ad(old_next_checkpoint_lsn <= log_sys->next_checkpoint_lsn);
+#endif /* UNIV_DEBUG */
 	mach_write_to_8(buf + LOG_CHECKPOINT_NO, log_sys->next_checkpoint_no);
 	mach_write_to_8(buf + LOG_CHECKPOINT_LSN, log_sys->next_checkpoint_lsn);
 
@@ -1911,6 +1948,7 @@ log_checkpoint(
 		return(false);
 	}
 
+	ut_ad(oldest_lsn >= log_sys->next_checkpoint_lsn);
 	log_sys->next_checkpoint_lsn = oldest_lsn;
 	log_write_checkpoint_info(sync);
 	ut_ad(!log_mutex_own());
@@ -2238,16 +2276,25 @@ loop:
 	/* At this point only page_cleaner should be active. We wait
 	here to let it complete the flushing of the buffer pools
 	before proceeding further. */
+	os_rmb;
+	ut_ad(buf_lru_manager_running_threads == srv_buf_pool_instances
+	      || buf_lru_manager_running_threads == 0);
 	srv_shutdown_state = SRV_SHUTDOWN_FLUSH_PHASE;
 	count = 0;
-	while (buf_page_cleaner_is_active) {
-		++count;
-		os_thread_sleep(100000);
-		if (srv_print_verbose_log && count > 600) {
+	while (buf_page_cleaner_is_active
+	       || buf_lru_manager_running_threads > 0) {
+
+		if (srv_print_verbose_log && count == 0) {
 			ib::info() << "Waiting for page_cleaner to"
 				" finish flushing of buffer pool";
+		}
+		++count;
+		os_thread_sleep(100000);
+		if (count > 600) {
 			count = 0;
 		}
+
+		os_rmb;
 	}
 
 	log_mutex_enter();
@@ -2310,7 +2357,7 @@ loop:
 
 		/* Wake the log tracking thread which will then immediatelly
 		quit because of srv_shutdown_state value */
-		if (srv_track_changed_pages) {
+		if (srv_redo_log_thread_started) {
 			os_event_reset(srv_redo_log_tracked_event);
 			os_event_set(srv_checkpoint_completed_event);
 		}
@@ -2379,7 +2426,7 @@ loop:
 	srv_shutdown_state = SRV_SHUTDOWN_LAST_PHASE;
 
 	/* Signal the log following thread to quit */
-	if (srv_track_changed_pages) {
+	if (srv_redo_log_thread_started) {
 		os_event_reset(srv_redo_log_tracked_event);
 		os_event_set(srv_checkpoint_completed_event);
 	}
@@ -2392,6 +2439,7 @@ loop:
 	ut_a(freed);
 
 	ut_a(lsn == log_sys->lsn);
+	ut_ad(lsn == log_sys->last_checkpoint_lsn);
 
 	if (lsn < srv_start_lsn) {
 		ib::error() << "Log sequence number at shutdown " << lsn
@@ -2405,8 +2453,6 @@ loop:
 		fil_write_flushed_lsn(lsn);
 	}
 
-	buf_parallel_dblwr_destroy();
-
 	fil_close_all_files();
 
 	/* Make some checks that the server really is quiet */
@@ -2579,6 +2625,7 @@ log_shutdown(void)
 	rw_lock_free(&log_sys->checkpoint_lock);
 
 	mutex_free(&log_sys->mutex);
+	mutex_free(&log_sys->write_mutex);
 	mutex_free(&log_sys->log_flush_order_mutex);
 
 	recv_sys_close();
