@@ -335,7 +335,7 @@ DECLARE_THREAD(io_handler_thread)(
 	The thread actually never comes here because it is exited in an
 	os_event_wait(). */
 
-	os_thread_exit(NULL);
+	os_thread_exit();
 
 	OS_THREAD_DUMMY_RETURN;
 }
@@ -345,7 +345,7 @@ DECLARE_THREAD(io_handler_thread)(
 /*********************************************************************//**
 Creates a log file.
 @return DB_SUCCESS or error code */
-static __attribute__((nonnull, warn_unused_result))
+static MY_ATTRIBUTE((nonnull, warn_unused_result))
 dberr_t
 create_log_file(
 /*============*/
@@ -542,7 +542,7 @@ create_log_files_rename(
 /*********************************************************************//**
 Opens a log file.
 @return DB_SUCCESS or error code */
-static __attribute__((nonnull, warn_unused_result))
+static MY_ATTRIBUTE((nonnull, warn_unused_result))
 dberr_t
 open_log_file(
 /*==========*/
@@ -1112,12 +1112,9 @@ srv_start_wait_for_purge_to_start()
 	}
 }
 
-/*********************************************************************//**
-Initializes the log tracking subsystem and starts its thread.  */
-static
+/** Initializes the log tracking subsystem and starts its thread.  */
 void
-init_log_online(void)
-/*=================*/
+srv_init_log_online(void)
 {
 	if (UNIV_UNLIKELY(srv_force_recovery > 0 || srv_read_only_mode)) {
 		srv_track_changed_pages = FALSE;
@@ -1736,7 +1733,8 @@ innobase_start_or_create_for_mysql(void)
 					strlen(fil_path_to_mysql_datadir)
 					+ 20 + sizeof "/innodb_status."));
 
-			sprintf(srv_monitor_file_name, "%s/innodb_status.%lu",
+			sprintf(srv_monitor_file_name,
+				"%s/innodb_status." ULINTPF,
 				fil_path_to_mysql_datadir,
 				os_proc_get_number());
 
@@ -1885,11 +1883,13 @@ innobase_start_or_create_for_mysql(void)
 				 NULL);
 	}
 
-	buf_lru_manager_is_active = true;
-
 	/* Make sure page cleaner is active. */
-	while (!buf_page_cleaner_is_active) {
+	os_rmb;
+	while (!buf_page_cleaner_is_active
+	       || buf_lru_manager_running_threads < srv_buf_pool_instances) {
+
 		os_thread_sleep(10000);
+		os_rmb;
 	}
 
 	srv_start_state_set(SRV_START_STATE_IO);
@@ -2153,7 +2153,7 @@ files_checked:
 	if (create_new_db) {
 
 		ut_a(!srv_read_only_mode);
-		init_log_online();
+		srv_init_log_online();
 
 		mtr_start(&mtr);
 
@@ -2277,8 +2277,6 @@ files_checked:
 		if (err != DB_SUCCESS)
 			return(srv_init_abort(err));
 
-		init_log_online();
-
 		purge_queue = trx_sys_init_at_db_start();
 
 		if (srv_force_recovery < SRV_FORCE_NO_LOG_REDO) {
@@ -2365,7 +2363,8 @@ files_checked:
 		server could crash in middle of key rotation. Some tablespace
 		didn't complete key rotation. Here, we will resume the
 		rotation. */
-		if (srv_force_recovery < SRV_FORCE_NO_LOG_REDO) {
+		if (!srv_read_only_mode
+		    && srv_force_recovery < SRV_FORCE_NO_LOG_REDO) {
 			fil_encryption_rotate();
 		}
 
