@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2008, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -310,11 +310,11 @@ sync_cell_get_event(
 
 	} else if (type == RW_LOCK_X_WAIT) {
 
-		return(cell->latch.lock->wait_ex_event);
+		return(&cell->latch.lock->wait_ex_event);
 
 	} else { /* RW_LOCK_S and RW_LOCK_X wait on the same event */
 
-		return(cell->latch.lock->event);
+		return(&cell->latch.lock->event);
 	}
 }
 
@@ -584,17 +584,18 @@ sync_array_cell_print(
 		}
 
 		fprintf(file,
-			"number of readers %lu, waiters flag %lu,"
-			" lock_word: %lx\n"
+			"number of readers " ULINTPF
+			", waiters flag " ULINTPF
+			", lock_word: %lx\n"
 			"Last time read locked in file %s line %lu\n"
 			"Last time write locked in file %s line %lu\n",
-			(ulong) rw_lock_get_reader_count(rwlock),
-			(ulong) rwlock->waiters,
-			rwlock->lock_word,
+			rw_lock_get_reader_count(rwlock),
+			rwlock->waiters,
+			static_cast<ulong>(rwlock->lock_word),
 			innobase_basename(rwlock->last_s_file_name),
-			(ulong) rwlock->last_s_line,
+			static_cast<ulong>(rwlock->last_s_line),
 			rwlock->last_x_file_name,
-			(ulong) rwlock->last_x_line);
+			static_cast<ulong>(rwlock->last_x_line));
 	} else {
 		ut_error;
 	}
