@@ -26,9 +26,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
 #include "hatoku_hton.h"
 
-#if TOKUDB_CHECK_JEMALLOC
 #include <dlfcn.h>
-#endif
 
 #include "my_tree.h"
 
@@ -281,7 +279,6 @@ static int tokudb_init_func(void *p) {
     db_env = NULL;
     tokudb_hton = (handlerton *) p;
 
-#if TOKUDB_CHECK_JEMALLOC
     if (tokudb::sysvars::check_jemalloc) {
         typedef int (*mallctl_type)(
             const char*,
@@ -308,7 +305,6 @@ static int tokudb_init_func(void *p) {
             goto error;
         }
     }
-#endif
 
     r = tokudb_set_product_name();
     if (r) {
@@ -407,6 +403,14 @@ static int tokudb_init_func(void *p) {
     db_env->set_errcall(db_env, tokudb_print_error);
     db_env->set_errpfx(db_env, tokudb_hton_name);
 
+    // Handle deprecated options
+    if (!tokudb::sysvars::support_xa(NULL)) {
+        TOKUDB_TRACE("Using tokudb_support_xa is deprecated and the "
+            "parameter may be removed in future releases. "
+            "See documentation and release notes for details");
+           tokudb::sysvars::set_support_xa(NULL, TRUE);
+    }
+
     //
     // set default comparison functions
     //
@@ -536,11 +540,14 @@ static int tokudb_init_func(void *p) {
     db_env->change_fsync_log_period(db_env, tokudb::sysvars::fsync_log_period);
 
     db_env->set_lock_timeout_callback(db_env, tokudb_lock_timeout_callback);
+    db_env->set_dir_per_db(db_env, tokudb::sysvars::dir_per_db);
 
     db_env->set_loader_memory_size(
         db_env,
         tokudb_get_loader_memory_size_callback);
 
+    db_env->set_check_thp(db_env, tokudb::sysvars::check_jemalloc);
+
     r = db_env->open(
         db_env,
         tokudb_home,
